# -*- encoding:utf-8 -*-
import json
import re
import time
from collections import defaultdict

from flask_app.general.商务技术评分提取 import combine_technical_and_business, parse_json_with_duplicates, \
    process_data_based_on_key, reorganize_data
from flask_app.general.通义千问long import upload_file, qianwen_long

def combine_evaluation_standards(truncate_file):
    # 定义默认的评审结果字典
    DEFAULT_EVALUATION_REVIEW = {
        "技术评分": "",
        "商务评分": ""
    }
    # 如果 truncate_file 是空字符串，直接返回包含空字符串的字典
    if not truncate_file:
        return DEFAULT_EVALUATION_REVIEW.copy()
    try:
        # 上传文件并获取文件ID
        file_id = upload_file(truncate_file)

        # 定义用户查询
        user_query1 = (
            "根据该文档，你判断它是否有关于技术评分或商务评分或投标报价的具体的评分及要求，如果有，返回'是'，否则返回'否'。"
        )  # 应对竞争性谈判这种无评分要求的情况
        # 执行查询
        judge_res = qianwen_long(file_id, user_query1)
        # 默认 judge 为 True
        judge = True
        # 检查 judge_res 的内容
        if '否' in judge_res:
            judge = False
        if judge:
            # 执行 user_query 相关的逻辑
            # user_query = "根据该文档中的评标办法前附表或者评分标准表，请你列出该文件的技术评分，商务评分，投标报价评审标准以及它们对应的具体评分要求，外层键名分别为'技术评分','商务评分','投标报价'。如果评分内容不是这3个，则返回文档中给定的评分内容以及它的评分要求，都以json的格式返回结果，如果该采购活动有多个包，则最外层键名为对应的包名。请不要回答有关资格审查的内容"
            user_query = (
                """
你是一个对招投标业务非常熟悉的专家。根据该文档中的评标办法表格，请你列出该文件的技术评分，商务评分，投标报价评分以及它们对应的具体评分要求，请以JSON格式返回结果。
格式要求：
    请以 JSON 格式返回结果，最外层键名为 '技术评分'、'商务评分' 和 '投标报价评分'。在每大项下，用键值对表示具体评分项，键为具体的评审因素，若评审因素存在嵌套（表格中存在层级），请使用嵌套键值对表示，外层键名为主评审因素，嵌套的子评审因素作为内层键名，最内键值为列表，列表中包含描述评分及要求的字典。每个字典的键包括：
        '评分'：具体得分或定性指标（如 '合格制'），无评分时可删去'评分'键值对。
        '要求'：说明评分标准。
    若这三大项评分中存在额外信息（不属于某个评审因素，即该大项评分的整体要求），在该评分项内部新增键名为'备注' ，值为该要求；请勿擅自添加不属于'评审因素'的键名。

要求与指南：
    1. 请首先定位评分细则的表格，不要回答有关资格审查的内容，也不要从评标办法正文中提取回答
    2. 你无需将表格的单元格内的内容进行拆分，需要将它视为一个整体
    3. '评分'的键值不能是一个范围数字，如'0-5分'，应该是一个具体数字，如'5分'，或者是一个定性的指标如'合格制'
    4. 如果该招标活动有多个包，则最外层键名为对应的包名,否则最外层键名为各大评分项
    5. 若表格中商务和技术评分混合一起，请你手动将它们区别，商务评分通常包含'售后服务'、'质量保证'、'业绩'、'企业人员'、'企业信用'等商务因素。
    6. 若表中的评分大项不是这三个，请你根据语义分别映射到'技术评分'、'商务评分'、'投标报价评分'，而不必严格按照表格中的名称，若大项的'xx评分'要求未在文中说明，则键名'xx评分'的键值设为'本项目无xx评分项'，例如"技术评分":"本项目无技术评分项" 

以下为示例输出，仅供格式参考：
    {
        "一包": {
            "技术评分": {
                "主要监理岗位的职责": [
                    {
                        "评分": "4分",
                        "要求": "1、总监理工程师的职责全面、清晰、合理得 1.2-2分；一般的1.2分。2、其他主要监理人员及岗位的职责全面、清晰、合理得 1.2-2分；一般的 1.2分。"
                    }
                ],
                "备注": "注：若不满足“与公安部、省公安厅、随州市公安局高清视频会议系统无缝对接互联互通”的要求，则本项技术部分（50分）不得分。"
            },
            "商务评分": {
                "控制系统内主板": [
                    {
                        "评分": "10分",
                        "要求": "所投电梯控制系统内主板为制造商原厂原品牌制造生产且为进口部件得 10分。（提供进口部件报关单及原产地证明扫描件加盖公章，否则不得分）"
                    }
                ],
                "制造商技术实力": [
                    {
                        "评分": "3分",
                        "要求": "一级证书得3分，二级证书得1分，其他不得分"
                    },
                    {
                        "评分": "2分",
                        "要求": "行业销量排名连续前 2 名，得 2 分，第 4-6 名得 0.5 分，其他不得分"
                    }
                ]
            },
            "投标报价评分": {
                "投标报价是否出现违反计价规范": [
                    {
                        "评分": "合格制",
                        "要求": "A:投标报价未违反计价规范的，评审意见为“合格”；B：投标报价违反计价规范的，评审意见为“不合格”"
                    }
                ]
            }
        }
    }
                """
            )
            # 执行第二个查询
            evaluation_res = qianwen_long(file_id, user_query)    #有些重复的键名，只有qianwen_long_text能保留
            # print(evaluation_res)
            # 清理和处理响应
            cleaned_evaluation_res = parse_json_with_duplicates(evaluation_res)   #处理重复键名的情况
            result_data = process_data_based_on_key(cleaned_evaluation_res)      #处理不知名外键的情况
            include = ['一包', '二包', '三包', '四包', '五包']
            target_values = ['技术', '设计', '实施']
            updated_jsons = {}

            # 检查是否有外层键匹配 include 列表
            if any(key for key in result_data if any(included in key for included in include)):  #检查result_data中的任何键是否包含include列表中的任意一个项。
                # 有匹配的项，处理这些项
                for key in result_data:
                    if any(item in key for item in include):
                        inner_dict = result_data[key]
                        updated_jsons[key] = combine_technical_and_business(inner_dict, target_values)    #对于分包，单独对分包内的'技术评分''商务评分'作处理
            else:
                # 没有匹配的项，对整个字典运行
                updated_jsons = combine_technical_and_business(result_data, target_values)
            final_res=reorganize_data(updated_jsons,include)                            #重新组织字典，尤其是分包的情况
            return final_res
        else:
            # 如果 judge 是 False，直接返回默认的技术标和商务标的结构
            result_data = {}
            result_data['技术评分'] = '本招标文件没有技术评分！'
            result_data['商务评分'] = '本招标文件没有商务评分！'
            return result_data

    except Exception as e:
        print(f"Error in combine_evaluation_standards: {e}")
        # 在出错时返回默认的包含空字符串的字典
        return DEFAULT_EVALUATION_REVIEW.copy()


if __name__ == "__main__":
    start_time=time.time()
    truncate_file=r"C:\Users\Administrator\Desktop\招标文件-采购类\tmp2\2024-新疆-塔城地区公安局食药环分局快检实验室项目_evaluation_method.pdf"
    # truncate_file = "C:\\Users\\Administrator\\Desktop\\货物标\\output2\\2-招标文件（统计局智能终端二次招标）_evaluation_method.pdf"
    # truncate_file="C:\\Users\\Administrator\\Desktop\\货物标\\output2\\广水市妇幼招标文件最新（W改）_evaluation_method.pdf"
    # truncate_file = "C:\\Users\\Administrator\\Desktop\\fsdownload\\2d481945-1f82-45a5-8e56-7fafea4a7793\\ztbfile_evaluation_method.pdf"
    # truncate_file="C:\\Users\\Administrator\\Desktop\\fsdownload\\ztbfile_evaluation_method.pdf"
    res = combine_evaluation_standards(truncate_file)
    print(json.dumps(res, ensure_ascii=False, indent=4))
    end_time=time.time()
    print("elapsed time:"+str(end_time-start_time))
